//
// Decompiled with: Decompiler.NET, Version=2.0.0.230, Culture=neutral, PublicKeyToken=null, Version: 2.0.0.230
// Decompilation Started at: 14/02/2008 11:24:41
// Copyright 2003 - 2004, Jungle Creatures, Inc., All Rights Reserved. http://www.junglecreatures.com/
// Written by Jonathan Pierce, Email: support@junglecreatures.com
//

namespace LanmarkControls.LECEngine

{
		
		#region Namespace Import Declarations
		
			using LanmarkControls.CommonLib.General;
			using LanmarkControls.CommonLib.Threading;
			using LanmarkControls.CTIJobManager;
			using LanmarkControls.LECHardwareAbstraction.FPGAInterface;
			using LanmarkControls.LECHardwareAbstraction;
			using System.Collections;
			using System.IO;
			using System;
			using System.Threading;
			
		#endregion
		
	public sealed class LECJobEngine
	
	{
		
		#region Fields
			private JobMsgThread applicationEventThread;
			private bool engineInitialized;
			private JobExceptMsgThread exceptionEventThread;
			private AutoResetEvent fpgaMessageAvailable;
			private static readonly LECJobEngine instanceLECJobEngine;
			private int jobIndex;
			private bool jobLoading;
			private Thread jobLoop;
			private bool jobRunning;
			private bool localJobAbort;
			private ExecuteMode loopMode;
			private JobMarkerMsgThread markerEventThread;
			private Thread messageDispatchThread;
			private string objectExtensionVersion;
			public event AsyncFPGAEventHandler OnAsyncFPGAEvent;
			public event AsyncFPGAExceptionEventHandler OnAsyncFPGAExceptionEvent;
			public event AsyncJobErrorMessageHandler OnAsyncJobErrorMessage;
			private bool stopDispatcher;
		#endregion
		
		#region Nested Types
		
			public enum ExecuteMode
			
			{
				ExecuteOnce = 0, 
				ExecuteContinuous = 1, 
			}
			
		#endregion
		
		#region Constructors
		
			static LECJobEngine ()
			
			{
				LECJobEngine.instanceLECJobEngine = new LECJobEngine ();
			}
			
			
			private LECJobEngine ()
			
			{
				this.fpgaMessageAvailable = new AutoResetEvent (false);
				this.jobIndex = -1;
				this.objectExtensionVersion = "";
				this.localJobAbort = false;
				this.stopDispatcher = false;
				this.applicationEventThread = ((JobMsgThread) null);
				this.exceptionEventThread = ((JobExceptMsgThread) null);
				this.markerEventThread = ((JobMarkerMsgThread) null);
				this.jobLoop = ((Thread) null);
				this.messageDispatchThread = ((Thread) null);
				this.engineInitialized = false;
				this.applicationEventThread = new JobMsgThread (this.fpgaMessageAvailable);
				this.exceptionEventThread = new JobExceptMsgThread (this.fpgaMessageAvailable);
				this.markerEventThread = new JobMarkerMsgThread (this.fpgaMessageAvailable);
			}
			
		#endregion
		
		#region Properties
		
			public JobOpStatus EngineStatus
			
			{
				get
				{
					if (this.jobLoading)
					{
						return JobOpStatus.Busy;
					}
					else if (! this.jobRunning)
					{
						return JobOpStatus.Idle;
					}
					else
					{
						return JobOpStatus.Busy;
					}
				}
			}
			
			
			public static LECJobEngine Instance
			
			{
				get
				{
					return LECJobEngine.instanceLECJobEngine;
				}
			}
			
			
			public string ObjectExtensionVersion
			
			{
				get
				{
					if (this.objectExtensionVersion == "")
					{
						this.objectExtensionVersion = LECJobManager.Instance.Version;
					}
					return this.objectExtensionVersion;
				}
				set
				{
					this.objectExtensionVersion = value;
				}
			}
			
		#endregion
		
		#region Methods
		
			public JobOpStatus AbortJob ()
			
			{
				this.localJobAbort = true;
				LECHAL.Instance.FPGAAbortState = true;
				return JobOpStatus.Idle;
			}
			
			public void ClearJobList ()
			
			{
				LECJobManager.Instance.ClearJobList ();
			}
			
			private void CompleteAbortProcess ()
			
			{
				LECHAL.ResetType resetType1 = LECHAL.Instance.ResetFPGA ();
				LECHAL.Instance.ClearFPGAEventQueue ();
				LECHAL.Instance.ClearFPGAExceptionEventQueue ();
				LECHAL.Instance.FPGAAbortState = false;
				this.localJobAbort = false;
			}
			
			public JobOpStatus ExecuteAllObjects (string fileName, ExecuteMode executeMode)
			
			{
				this.jobIndex = LECJobManager.Instance.IndexOf (fileName);
				if (this.jobIndex < 0)
				{
					return JobOpStatus.NoJob;
				}
				else if (LECJobManager.Instance.GetObjectCount (this.jobIndex) == 0)
				{
					return JobOpStatus.NoJob;
				}
				else if (this.EngineStatus == JobOpStatus.Busy)
				{
					return JobOpStatus.Busy;
				}
				else
				{
					this.loopMode = executeMode;
					this.jobLoop = new Thread (new ThreadStart (this.ExecuteJobLoop));
					this.jobLoop.Start ();
					return JobOpStatus.Busy;
				}
			}
			
			private void ExecuteJobLoop ()
			
			{
				ObjectRunStatus objectRunStatus1;
				int i1;
				int i3;
				int i4;
				int i5;
				this.jobLoading = true;
				while (true)
				{
					this.jobRunning = true;
					i1 = LECJobManager.Instance.GetObjectCount (this.jobIndex);
					for (int i2 = 0; (i2 < i1); i2++)
					{
						Thread.Sleep (0);
						i3 = LECJobManager.Instance.GetAlignGroupMembership (this.jobIndex, i2);
						if (i3 <= 0)
						{
							objectRunStatus1 = ((ObjectRunStatus) LECJobManager.Instance.ExecuteObject (this.jobIndex, i2));
							if (objectRunStatus1 != ObjectRunStatus.Success)
							{
								this.ProcessRunError (objectRunStatus1, LECJobManager.Instance.GetObjectName (this.jobIndex, i2));
								return;
							}
						}
						else
						{
							for (i4 = LECJobManager.Instance.GetAlignRepeatCount (this.jobIndex, i2); (i4 > 0); i4--)
							{
								for (i5 = 0; (i5 < i1); i5++)
								{
									if (LECJobManager.Instance.GetAlignGroupMembership (this.jobIndex, i5) == i3)
									{
										LECJobManager.Instance.DoStepRepeat (i4, this.jobIndex, i5);
										objectRunStatus1 = ((ObjectRunStatus) LECJobManager.Instance.ExecuteObject (this.jobIndex, i5));
										if (objectRunStatus1 != ObjectRunStatus.Success)
										{
											this.ProcessRunError (objectRunStatus1, LECJobManager.Instance.GetObjectName (this.jobIndex, i5));
											return;
										}
									}
								}
							}
							i2 += i4;
						}
					}
					while (this.jobRunning)
					{
						if (! this.localJobAbort)
						{
							Thread.Sleep (0);
							continue;
						}
						else
						{
							this.ProcessRunError (ObjectRunStatus.Abort, "Job Engine");
							return;
						}
					}
					if (this.loopMode != ExecuteMode.ExecuteContinuous)
					{
						this.jobLoading = false;
						this.jobIndex = -1;
						return;
					}
				}
			}
			
			public bool Initialize ()
			
			{
				LECJobManager.Instance.CacheFonts ();
				this.stopDispatcher = false;
				bool b1 = this.fpgaMessageAvailable.Reset ();
				if ((this.applicationEventThread != null) && (! this.applicationEventThread.StartThread ()))
				{
					return false;
				}
				else if ((this.exceptionEventThread != null) && (! this.exceptionEventThread.StartThread ()))
				{
					return false;
				}
				else if ((this.markerEventThread != null) && (! this.markerEventThread.StartThread ()))
				{
					return false;
				}
				else
				{
					this.messageDispatchThread = new Thread (new ThreadStart (this.MessageEventLoop));
					this.messageDispatchThread.Start ();
					LECHAL.Instance.EnableFPGAEventQueue (true);
					this.engineInitialized = true;
					return true;
				}
			}
			
			public JobOpStatus LoadJobFromFile (string fileName)
			
			{
				if (LECJobManager.Instance.IndexOf (Path.GetFileName (fileName)) >= 0)
				{
					return JobOpStatus.Success;
				}
				int i1 = LECJobManager.Instance.LoadFromFile (fileName);
				if (i1 == 0)
				{
					return JobOpStatus.Success;
				}
				else
				{
					return ((JobOpStatus) i1);
				}
			}
			
			private void MessageEventLoop ()
			
			{
				Exception exception1;
				ArrayList arrayList1 = new ArrayList ();
				try
				{
					while (! this.stopDispatcher)
					{
						arrayList1.Clear ();
						if (this.exceptionEventThread != null)
						{
							while (this.exceptionEventThread.Content.Count > 0)
							{
								int i1 = arrayList1.Add (this.exceptionEventThread.Content.Dequeue ());
							}
						}
						if (this.markerEventThread != null)
						{
							while (this.markerEventThread.Content.Count > 0)
							{
								int i2 = arrayList1.Add (this.markerEventThread.Content.Dequeue ());
							}
						}
						if (this.applicationEventThread != null)
						{
							while (this.applicationEventThread.Content.Count > 0)
							{
								int i3 = arrayList1.Add (this.applicationEventThread.Content.Dequeue ());
							}
						}
						if (arrayList1.Count == 0)
						{
							bool b1 = this.fpgaMessageAvailable.WaitOne ();
							if (this.stopDispatcher)
							{
								return;
							}
							else
							{
								if (this.exceptionEventThread != null)
								{
									while (this.exceptionEventThread.Content.Count > 0)
									{
										int i4 = arrayList1.Add (this.exceptionEventThread.Content.Dequeue ());
									}
								}
								if (this.markerEventThread != null)
								{
									while (this.markerEventThread.Content.Count > 0)
									{
										int i5 = arrayList1.Add (this.markerEventThread.Content.Dequeue ());
									}
								}
								if (this.applicationEventThread != null)
								{
									while (this.applicationEventThread.Content.Count > 0)
									{
										int i6 = arrayList1.Add (this.applicationEventThread.Content.Dequeue ());
									}
								}
							}
						}
						if (arrayList1.Count > 0)
						{
							this.ProcessEventMessages (arrayList1);
						}
						if (this.stopDispatcher)
						{
							return;
						}
					}
				}
				catch (Exception exception2)
				{
					exception1 = exception2;
					Logging.Instance.Log (0, "EventThread.Action() - Exception thrown. Abort.");
					Logging.Instance.Log (0, exception1.ToString ());
				}
			}
			
			private void ProcessEventMessages (ArrayList plstEvents)
			
			{
				MecExceptionMessage mecExceptionMessage1;
				int i1;
				int i2;
				MecMessage mecMessage1;
				if (plstEvents.Count == 0)
				{
					return;
				}
				bool b1 = false;
				foreach (object object1 in plstEvents)
				{
					if (object1 != null)
					{
						if (! (object1 is MecExceptionMessage))
						{
							mecMessage1 = (object1 as MecMessage);
							if (mecMessage1.MessageID == 65)
							{
								this.jobRunning = true;
							}
							if (mecMessage1.MessageID == 66)
							{
								this.jobRunning = false;
							}
							if (this.OnAsyncFPGAEvent != null)
							{
								this.OnAsyncFPGAEvent (mecMessage1.MessageID, mecMessage1.MessageOperand);
							}
							continue;
						}
						mecExceptionMessage1 = (object1 as MecExceptionMessage);
						b1 |= (mecExceptionMessage1.ExceptionCode == 8001);
						b1 |= (mecExceptionMessage1.ExceptionCode == 9001);
						i1 = (20544 | ((int) (((long) (mecExceptionMessage1.ExceptionCode << 16)) & ((long) (-65536)))));
						i2 = ((int) mecExceptionMessage1.LastOpcode);
						if (this.OnAsyncFPGAEvent != null)
						{
							this.OnAsyncFPGAExceptionEvent (i1, i2);
						}
						continue;
					}
				}
				if (b1)
				{
					LECHAL.Instance.StartCommandProcessing (false);
				}
			}
			
			private void ProcessRunError (ObjectRunStatus runStatus, string objectName)
			
			{
				this.CompleteAbortProcess ();
				if (this.OnAsyncJobErrorMessage != null)
				{
					this.OnAsyncJobErrorMessage (runStatus.ToString (), objectName);
				}
				this.jobLoading = false;
				this.jobRunning = false;
			}
			
			public JobOpStatus RemoveJob (string fileName)
			
			{
				int i1 = LECJobManager.Instance.IndexOf (Path.GetFileName (fileName));
				if (i1 < 0)
				{
					return JobOpStatus.NoJob;
				}
				else
				{
					LECJobManager.Instance.RemoveJobAt (i1);
					return JobOpStatus.Success;
				}
			}
			
			public void Shutdown ()
			
			{
				if (this.engineInitialized)
				{
					LECJobManager.Instance.ClearJobList ();
					LECJobManager.Instance.ReleaseFontCache ();
					LECHAL.Instance.EnableFPGAEventQueue (false);
					this.stopDispatcher = true;
					bool b1 = this.fpgaMessageAvailable.Set ();
					if ((this.markerEventThread != null) && this.markerEventThread.IsThreadRunning)
					{
						bool b2 = this.markerEventThread.StopThread ();
					}
					if ((this.applicationEventThread != null) && this.applicationEventThread.IsThreadRunning)
					{
						bool b3 = this.applicationEventThread.StopThread ();
					}
					if ((this.exceptionEventThread != null) && this.exceptionEventThread.IsThreadRunning)
					{
						bool b4 = this.exceptionEventThread.StopThread ();
					}
					this.engineInitialized = false;
					GC.Collect ();
					GC.WaitForPendingFinalizers ();
				}
			}
			
		#endregion
	}
	
}

